@@ -20,7 +20,6 @@
 
 #include "cd-recorder.h"
 #include "cdrecorder-marshal.h"
-#include "make-iso.h"
 
 #undef DEBUG_CDRECORD
 
@@ -49,10 +48,12 @@ struct CDRecorderPrivate {
 	gboolean changed_text;
 	gboolean send_return;
 	gboolean expect_cdrecord_to_die;
-	gint music_count;
 	GList *actions;
 
 	char *last_error;
+	
+	GList *tracks;
+	int track_count;
 };
 
 /* Signals */
@@ -73,14 +74,14 @@ static GObjectClass *parent_class = NULL;
 static void cd_recorder_class_init (CDRecorderClass *class);
 static void cd_recorder_init       (CDRecorder      *cdrecorder);
 
-static int cd_recorder_write_iso_cdrecord (CDRecorder *cdrecorder,
-		CDDrive *recorder, const char *filename,
+static int cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
+		CDDrive *recorder, GList *tracks,
 		int speed, gboolean eject, gboolean blank,
-		gboolean music, gboolean dummy_write);
-static int cd_recorder_write_iso_growisofs (CDRecorder *cdrecorder,
-		CDDrive *recorder, const char *filename,
+		gboolean dummy_write);
+static int cd_recorder_write_growisofs (CDRecorder *cdrecorder,
+		CDDrive *recorder, GList *tracks,
 		int speed, gboolean eject, gboolean blank,
-		gboolean music, gboolean dummy_write);
+		gboolean dummy_write);
 
 GType
 cd_recorder_get_type (void)
@@ -250,16 +251,8 @@ cdrecord_stdout_read (GIOChannel   *source,
 					       cd_recorder_table_signals[ACTION_CHANGED], 0,
 					       WRITING_CD);
 			}
-			if (cdrecorder->priv->music_count == 0) {
-				percent = (mb_total > 0) ? ((double)mb_written/mb_total) * 0.98 : 0.0;
-			} else {			
-				if (mb_total <= 0) {
-					percent = 0.0;
-				} else {
-					percent = (double)((track-1)/(double)cdrecorder->priv->music_count)
-					 +	((double)mb_written/mb_total) * 0.98 / (double)cdrecorder->priv->music_count;
-				}
-			}
+			percent = 0.98 * ((double)((track-1)/(double)cdrecorder->priv->track_count)
+					  + ((double)mb_written/mb_total) / (double)cdrecorder->priv->track_count);
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[PROGRESS_CHANGED], 0,
 				       percent);
@@ -416,66 +409,81 @@ cdrecord_stderr_read (GIOChannel   *source,
 }
 
 int
-cd_recorder_write_iso (CDRecorder *cdrecorder,
+cd_recorder_write_tracks (CDRecorder *cdrecorder,
 		       CDDrive *recorder,
-		       const char *filename,
+		       GList *tracks,
 		       int speed,
 		       gboolean eject,
 		       gboolean blank,
-		       gboolean music,
 		       gboolean dummy_write)
+		       /* TODO: GError **error */
 {
+	g_return_val_if_fail (tracks != NULL, RESULT_ERROR);
+	cdrecorder->priv->tracks = tracks;
+	cdrecorder->priv->track_count = g_list_length (tracks);
+
+	/* TODO: this could be changed so that cd_needs_growisofs()
+	   takes tracks, and will decide cdrecord/growisofs/error. */
 	if (cd_needs_growisofs (recorder)) {
-		return cd_recorder_write_iso_growisofs (cdrecorder,
-				recorder, filename, speed, eject, blank,
-				music, dummy_write);
+		return cd_recorder_write_growisofs (cdrecorder,
+						    recorder, tracks, speed,
+						    eject, blank, dummy_write);
 	} else {
-		return cd_recorder_write_iso_cdrecord (cdrecorder,
-				recorder, filename, speed, eject, blank,
-				music, dummy_write);
+		return cd_recorder_write_cdrecord (cdrecorder,
+						   recorder, tracks, speed,
+						   eject, blank, dummy_write);
 	}
 }
 
 static int
-cd_recorder_write_iso_growisofs (CDRecorder *cdrecorder,
+cd_recorder_write_growisofs (CDRecorder *cdrecorder,
 				 CDDrive *recorder,
-				 const char *filename,
+				 GList *tracks,
 				 int speed,
 				 gboolean eject,
 				 gboolean blank,
-				 gboolean music,
 				 gboolean dummy_write)
 {
-	const char *argv[200]; /* Shouldn't need more than 200 arguments */
+	GPtrArray *argv;
 	char *speed_str, *dev_str;
 	int stdout_pipe, stderr_pipe;
 	guint stdout_tag, stderr_tag;
 	GIOChannel *channel;
 	GError *error;
-	int i;
+	Track *t;
 
-	i = 0;
-
-	argv[i++] = "growisofs";
-	speed_str = g_strdup_printf ("-speed=%d", speed);
-	if (speed != 0) {
-		argv[i++] = speed_str;
+	if (g_list_length (tracks) != 1) {
+		g_warning ("Can only use growisofs on a single track");
+		return RESULT_ERROR;
+	}
+	t = (Track*)tracks->data;
+	if (t->type != TRACK_TYPE_DATA) {
+		g_warning ("Can only use growisofs on a data track");
+		return RESULT_ERROR;
 	}
 
-	argv[i++] = "-dvd-compat";
+	argv = g_ptr_array_new ();
+
+	g_ptr_array_add (argv, "growisofs");
+	speed_str = g_strdup_printf ("-speed=%d", speed);
+	if (speed != 0) {
+		g_ptr_array_add (argv, speed_str);
+	}
+
+	g_ptr_array_add (argv, "-dvd-compat");
 
 	/* Weird, innit? We tell growisofs we have a tty so it ignores
 	 * the fact that the DVD+ has an ISO fs already */
 	if (blank) {
-		argv[i++] = "-use-the-force-luke=tty";
+		g_ptr_array_add (argv, "-use-the-force-luke=tty");
 	}
 
-	argv[i++] = "-Z";
+	g_ptr_array_add (argv, "-Z");
 
-	dev_str = g_strdup_printf ("%s=%s", recorder->device, filename);
-	argv[i++] = dev_str;
+	dev_str = g_strdup_printf ("%s=%s", recorder->device, t->data.filename);
+	g_ptr_array_add (argv, dev_str);
 
-	argv[i++] = NULL;
+	g_ptr_array_add (argv, NULL);
 
 	cdrecorder->priv->cdr_stderr = NULL;
 retry_growisofs:
@@ -503,16 +511,19 @@ retry_growisofs:
 			cancel_cdrecord, FALSE);
 
 #ifdef DEBUG_CDRECORD
-	g_print ("launching command: ");
-	for (i = 0; argv[i] != NULL; i++) {
-		g_print ("%s ", argv[i]);
+	{
+		int i;
+		g_print ("launching command: ");
+		for (i = 0; argv->len; i++) {
+			g_print ("%s ", (char*)g_ptr_array_index (argv, i));
+		}
+		g_print ("\n");
 	}
-	g_print ("\n");
 #endif
 
 	error = NULL;
 	if (!g_spawn_async_with_pipes  (NULL,
-				        (char **)argv,
+					(char**)argv->pdata,
 					NULL,
 					G_SPAWN_SEARCH_PATH,
 					NULL, NULL,
@@ -521,9 +532,10 @@ retry_growisofs:
 					&stdout_pipe,
 					&stderr_pipe,
 					&error)) {
+		int i;
 		g_warning ("growisofs command failed: %s\n", error->message);
-		for (i = 0; argv[i] != NULL; i++) {
-			g_print ("%s ", argv[i]);
+		for (i = 0; argv->len; i++) {
+			g_print ("%s ", (char*)g_ptr_array_index (argv, i));
 		}
 		g_print ("\n");
 		g_error_free (error);
@@ -567,7 +579,8 @@ retry_growisofs:
 
 	g_free (speed_str);
 	g_free (dev_str);
-
+	g_ptr_array_free (argv, TRUE);
+	
 	g_signal_emit (G_OBJECT (cdrecorder),
 			cd_recorder_table_signals[ANIMATION_CHANGED], 0,
 			FALSE);
@@ -576,56 +589,65 @@ retry_growisofs:
 }
 
 static int
-cd_recorder_write_iso_cdrecord (CDRecorder *cdrecorder,
+cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 		       CDDrive *recorder,
-		       const char *filename,
+		       GList *tracks,
 		       int speed,
 		       gboolean eject,
 		       gboolean blank,
-		       gboolean music,
 		       gboolean dummy_write)
 {
-	const char *argv[200]; /* Shouldn't need more than 200 arguments */
+	GPtrArray *argv;
 	char *speed_str, *dev_str;
 	int stdout_pipe, stderr_pipe;
 	guint stdout_tag, stderr_tag;
 	GIOChannel *channel;
 	GError *error;
-	int i, saved_i;
+	GList *l;
 
-	i = 0;
-	saved_i = 0;
+	g_return_val_if_fail (tracks != NULL, RESULT_ERROR);
 
-	argv[i++] = "cdrecord";
+	argv = g_ptr_array_new ();
+	g_ptr_array_add (argv, "cdrecord");
 
 	speed_str = g_strdup_printf ("speed=%d", speed);
 	if (speed != 0) {
-		argv[i++] = speed_str;
+		g_ptr_array_add (argv, speed_str);
 	}
 	dev_str = g_strdup_printf ("dev=%s", recorder->cdrecord_id);
-	argv[i++] = dev_str;
+	g_ptr_array_add (argv, dev_str);
 	if (dummy_write) {
-		argv[i++] = "-dummy";
+		g_ptr_array_add (argv, "-dummy");
 	}
 	if (eject) {
-		argv[i++] = "-eject";
+		g_ptr_array_add (argv, "-eject");
 	}
 	if (blank) {
-		argv[i++] = "blank=fast";
-	}
-	argv[i++] = "-v";
-	if (music) {
-		argv[i++] = "-pad";
-		argv[i++] = "-audio";
-		saved_i = i;
-		cdrecorder->priv->music_count = build_list_of_music_arguments ("burn:///", &i, argv, 200-i-5);
-	} else {
-		cdrecorder->priv->music_count = 0;
-		argv[i++] = "-data";
-		argv[i++] = filename;
+		g_ptr_array_add (argv, "blank=fast");
 	}
+	g_ptr_array_add (argv, "-v");
 
-	argv[i++] = NULL;
+	l = tracks;
+	while (l != NULL && l->data != NULL) {
+		Track *track = l->data;
+		switch (track->type) {
+		case TRACK_TYPE_DATA:
+			g_ptr_array_add (argv, "-data");
+			g_ptr_array_add (argv, "-nopad"); /* TODO? */
+			g_ptr_array_add (argv, track->data.filename);
+			break;
+		case TRACK_TYPE_AUDIO:
+			g_ptr_array_add (argv, "-audio");
+			g_ptr_array_add (argv, "-pad");
+			g_ptr_array_add (argv, track->audio.filename);
+			/* TODO: handle CD-TEXT somehow */
+			break;
+		default:
+			g_warning ("Unknown track type %d", track->type);
+		}
+		l = g_list_next (l);
+	}
+	g_ptr_array_add (argv, NULL);
 
 	cdrecorder->priv->cdr_stderr = NULL;
  retry:
@@ -653,16 +675,20 @@ cd_recorder_write_iso_cdrecord (CDRecorder *cdrecorder,
 		       cancel_cdrecord, FALSE);
 
 #ifdef DEBUG_CDRECORD
-	g_print ("launching command: ");
-	for (i = 0; argv[i] != NULL; i++) {
-		g_print ("%s ", argv[i]);
+	{
+		guint i;
+		g_print ("launching command: ");
+		for (i = 0; i < argv->len; i++) {
+			g_print ("%s ", (char*)g_ptr_array_index (argv, i));
+		}
+		g_print ("\n");
+		return -1;
 	}
-	g_print ("\n");
 #endif
 
 	error = NULL;
 	if (!g_spawn_async_with_pipes  (NULL,
-					(char **)argv,
+					(char**)argv->pdata,
 					NULL,
 					G_SPAWN_SEARCH_PATH,
 					NULL, NULL,
@@ -713,13 +739,7 @@ cd_recorder_write_iso_cdrecord (CDRecorder *cdrecorder,
 
 	g_free (speed_str);
 	g_free (dev_str);
-
-	if (music) {
-		int j;
-		for (j=0;j<cdrecorder->priv->music_count;j++) {
-			g_free ((char *)argv[saved_i+j]);
-		}
-	}
+	g_ptr_array_free (argv, TRUE);
 
 	g_signal_emit (G_OBJECT (cdrecorder),
 		       cd_recorder_table_signals[ANIMATION_CHANGED], 0,
@@ -758,7 +778,7 @@ cd_recorder_finalize (GObject *object)
 	g_return_if_fail (object != NULL);
 
 	g_string_free (cdrecorder->priv->cdr_stderr, TRUE);
-	//FIXME cleanup
+	/* FIXME cleanup */
 
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
