@@ -14,6 +14,26 @@ enum {
 	RESULT_RETRY
 };
 
+typedef enum {
+  TRACK_TYPE_AUDIO,
+  TRACK_TYPE_DATA
+} TrackType;
+
+typedef struct Track Track;
+
+struct Track {
+  TrackType type;
+  union {
+    struct {
+      char *filename;
+      char *cdtext;
+    } audio;
+    struct {
+      char *filename;
+    } data;
+  };
+};
+
 #define CD_TYPE_RECORDER            (cd_recorder_get_type ())
 #define CD_RECORDER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), CD_TYPE_RECORDER, CDRecorder))
 #define CD_RECORDER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), CD_TYPE_RECORDER, CDRecorderClass))
@@ -52,9 +72,9 @@ CDRecorder     *cd_recorder_new			(void);
 
 GList	       *cd_recorder_list_actions	(CDRecorder *cdrecorder);
 
-int		cd_recorder_write_iso		(CDRecorder *cdrecorder,
+int		cd_recorder_write_tracks		(CDRecorder *cdrecorder,
 						 CDDrive *recorder,
-						 const gchar *filename,
+						 GList *tracks,
 						 gint speed,
 						 gboolean eject,
 						 gboolean blank,
