@@ -22,10 +22,12 @@
    Authors: Alexander Larsson <alexl@redhat.com>
 */
 #include <gtk/gtk.h>
+#include <gtk/gtkfilesel.h>
 #include <gtk/gtkseparatormenuitem.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <eel/eel-vfs-extensions.h>
+#include <gconf/gconf-client.h>
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -303,6 +305,95 @@ insert_rwcd_request_cb (CDRecorder *cdrecorder, gboolean is_reload,
 	}
 }
 
+static gboolean
+media_type_matches (CDMediaType type, gboolean blank_cd)
+{
+	switch (type)
+	{
+	case CD_MEDIA_TYPE_ERROR:
+		return FALSE;
+	case CD_MEDIA_TYPE_UNKNOWN:
+		return TRUE;
+	case CD_MEDIA_TYPE_CD:
+	case CD_MEDIA_TYPE_DVD:
+	case CD_MEDIA_TYPE_DVD_RAM:
+		return FALSE;
+	case CD_MEDIA_TYPE_CDR:
+		return (blank_cd ? FALSE : TRUE);
+	case CD_MEDIA_TYPE_CDRW:
+		return TRUE;
+	case CD_MEDIA_TYPE_DVDR:
+		return (blank_cd ? FALSE : TRUE);
+	case CD_MEDIA_TYPE_DVDRW:
+		return TRUE;
+	case CD_MEDIA_TYPE_DVD_PLUS_R:
+		return (blank_cd ? FALSE : TRUE);
+	case CD_MEDIA_TYPE_DVD_PLUS_RW:
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+wait_for_insertion (CDDrive *rec, gboolean blank_cd)
+{
+	CDMediaType type;
+	gboolean reload;
+
+	reload = FALSE;
+	type = guess_media_type (rec->device);
+
+	if (type == CD_MEDIA_TYPE_ERROR) {
+		reload = TRUE;
+	}
+
+	while (!media_type_matches (type, blank_cd)) {
+		GtkWidget *dialog, *parent;
+		const char *msg, *title;
+		int res;
+
+		if (reload && blank_cd) {
+			msg = N_("Please insert a rewritable media in the drive tray.");
+			title = N_("Insert rewritable media");
+		} else if (reload && !blank_cd) {
+			msg = N_("Please insert a blank media in the drive tray.");
+			title = N_("Insert blank media");
+		} else if (blank_cd) {
+			msg = N_("Please replace the in-drive media by a rewritable media.");
+			title = N_("Reload rewritable media");
+		} else {
+			msg = N_("Please replace the in-drive media by a blank media.");
+			title = N_("Reload blank media");
+		}
+
+		parent = glade_xml_get_widget (xml, "cdr_dialog");
+		dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_INFO,
+				GTK_BUTTONS_OK_CANCEL,
+				"%s", _(msg));
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+				GTK_RESPONSE_OK);
+		gtk_window_set_title (GTK_WINDOW (dialog), _(title));
+		res = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		if (res == GTK_RESPONSE_CANCEL) {
+			return FALSE;
+		}
+
+		type = guess_media_type (rec->device);
+		reload = FALSE;
+		if (type == CD_MEDIA_TYPE_UNKNOWN
+				|| type == CD_MEDIA_TYPE_ERROR) {
+			reload = TRUE;
+		}
+	}
+
+	return TRUE;
+}
+
+
 static void
 details_clicked (GtkButton *button, gpointer data)
 {
@@ -443,7 +534,7 @@ estimate_size (char *uri)
 	gnome_vfs_directory_visit (uri,
 				   GNOME_VFS_FILE_INFO_DEFAULT,
 				   GNOME_VFS_DIRECTORY_VISIT_DEFAULT,
-				   estimate_size_callback,
+				   (GnomeVFSDirectoryVisitFunc) estimate_size_callback,
 				   &size);
 
 	return size;
@@ -462,52 +553,270 @@ setup_close_button (void)
 	g_signal_connect (close_button, "clicked", (GCallback)gtk_main_quit, NULL);
 }
 
+static void
+track_free (Track *track, gpointer user_data)
+{
+	switch (track->type) {
+	case TRACK_TYPE_DATA:
+		g_free (track->data.filename);
+		break;
+	case TRACK_TYPE_AUDIO:
+		g_free (track->audio.filename);
+		g_free (track->audio.cdtext);
+		break;
+	default:
+		g_warning ("Invalid track type %d", track->type);
+	}
+}
+
+static gboolean 
+overwrite_existing_file (GtkWindow *filesel, const char *filename)
+{
+	GtkWidget *dialog;
+	gint ret;
+
+	dialog = gtk_message_dialog_new (filesel,
+					 GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_QUESTION,
+					 GTK_BUTTONS_NONE,
+					 _("A file named \"%s\" already exists.\nDo you want to overwrite it?"),
+					 filename);
+	/* Add Cancel button */
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+
+	/* Add Overwrite button */
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       _("_Overwrite"), GTK_RESPONSE_YES);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+	ret = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+
+	return (ret == GTK_RESPONSE_YES);
+}
+
+static void
+display_error_dialog (GtkWindow *win, const gchar *msg)
+{
+	GtkWidget *error_dlg;
+	
+	g_return_if_fail (msg != NULL);
+
+	error_dlg = gtk_message_dialog_new (win ? win : NULL,
+					    0,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    msg);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (error_dlg), GTK_RESPONSE_OK);
+	gtk_window_set_resizable (GTK_WINDOW (error_dlg), FALSE);
+	gtk_dialog_run (GTK_DIALOG (error_dlg));
+	gtk_widget_destroy (error_dlg);
+}
+
+static void
+file_sel_ok_cb (GtkWidget *widget, gpointer data)
+{
+	GtkFileSelection *fsel;
+	const gchar *file_name;
+	gint name_len;
+
+	fsel = (GtkFileSelection *)data;
+
+	file_name = gtk_file_selection_get_filename (fsel);
+
+	name_len = strlen (file_name);
+
+	if (name_len < 1)
+		return;
+
+	/* Change into directory if that's what user selected */
+	if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
+		gchar *dir_name;
+
+		if (file_name [name_len - 1] != '/') {
+			/* The file selector needs a '/' at the end of a 
+			 * directory name 
+			 */
+			dir_name = g_strconcat (file_name, "/", NULL);
+		} else {
+			dir_name = g_strdup (file_name);
+		}
+		gtk_file_selection_set_filename (fsel, dir_name);
+		g_free (dir_name);
+	} else {
+		if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
+			if (!overwrite_existing_file (GTK_WINDOW (fsel), file_name)) {
+				return;
+			}
+		}
+
+		gtk_widget_hide (GTK_WIDGET (fsel));
+		g_object_set_data (G_OBJECT (fsel), 
+				   "file-name", 
+				   g_strdup (file_name));
+		gtk_main_quit ();
+	}
+}
+
+static void
+file_sel_cancel_cb (GtkWidget *widget, gpointer data)
+{
+	gtk_widget_hide (GTK_WIDGET (data));
+	gtk_main_quit ();
+
+	g_object_set_data (G_OBJECT (data),
+			   "file-name",
+			   NULL);
+}
+
+static gint
+file_sel_delete_event_cb (GtkWidget *widget, GdkEventAny *event, gpointer data)
+{
+	gtk_widget_hide (widget);
+	gtk_main_quit ();
+
+	g_object_set_data (G_OBJECT (data),
+			   "file-name",
+			   NULL);
+	return TRUE;
+}
+
+static char *
+select_iso_filename (void)
+{
+	GtkWidget *filesel;
+	gpointer data;
+
+	filesel = gtk_file_selection_new (_("Choose a filename for the cdrom image"));
+	gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
+					 "~/image.iso");
+
+	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
+			  "clicked",
+			  G_CALLBACK(file_sel_ok_cb),
+			  filesel);
+	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->cancel_button),
+			  "clicked",
+			  G_CALLBACK(file_sel_cancel_cb),
+			  filesel);
+	g_signal_connect (G_OBJECT (filesel),
+			  "delete_event",
+			  G_CALLBACK(file_sel_delete_event_cb),
+			  filesel);
+
+	gtk_window_set_modal (GTK_WINDOW (filesel), TRUE);
+
+	gtk_widget_show (filesel);
+
+	/* Run in a loop untill we get the response from file selector */
+	gtk_main ();
+
+	data =  g_object_get_data (G_OBJECT (filesel), "file-name");
+
+	gtk_widget_destroy (filesel);
+
+	return (char *)data;
+}
+
 static int
 burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, gboolean music_cd, int speed, gboolean dummy)
 {
 	CDRecorder *cdrecorder;
-	GtkWidget *filesel;
+	GList *tracks;
 	char *filename, *str;
-	const char *f;
 	int fd;
 	int res = RESULT_ERROR;
 
+	tracks = NULL;
+	filename = NULL;
+
 	if (rec->type == CDDRIVE_TYPE_FILE) {
 		if (source_iso != NULL) {
 			/* TODO: display some message here */
 			g_print ("Writing ISO onto ISO, you freak!\n");
 			return RESULT_FINISHED;
 		}
-		filesel = gtk_file_selection_new (_("Choose a filename for the CD-ROM image"));
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
-						 "~/image.iso");
 
-		res = gtk_dialog_run (GTK_DIALOG (filesel));
-		f = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
-		filename = g_strdup (f);
-		gtk_widget_destroy (filesel);
-		
-		if (res == GTK_RESPONSE_CANCEL) {
+		/* Run the file selector and get the iso file name selected */
+		filename = select_iso_filename ();
+
+		/* No file selected, just return; Else proceed */
+		if (filename == NULL) {
 			return RESULT_CANCEL;
+		} else {
+
+			/* Check if you have permission to create or
+			 * overwrite the file.
+			 */
+			if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+				if (access (filename, W_OK) == -1) {
+					display_error_dialog (NULL, _("Don't have permissions to overwrite file."));
+					g_free (filename);
+					return RESULT_ERROR;
+				}
+			} else {
+				gchar *dir_name;
+				dir_name = g_path_get_dirname (filename);
+
+				if (access (dir_name, W_OK) == -1) {
+					display_error_dialog (NULL, _("Don't have permissions to create file."));
+					g_free (filename);
+					g_free (dir_name);
+					return RESULT_ERROR;
+				}
+				g_free (dir_name);
+			}
 		}
 	} else {
-		if (music_cd) {
-			filename = NULL;
-		} else if (source_iso == NULL) {
-			filename = g_build_filename (g_get_tmp_dir (), "image.iso.XXXXXX", NULL);
-			fd = mkstemp(filename);
-			close (fd);
-		} else {
+		if (source_iso == NULL && !music_cd) {
+			GConfClient *gc;
+			const char *path;
+
+			gc = gconf_client_get_default ();
+			path = gconf_client_get_string (gc, "/apps/nautilus-cd-burner/temp_iso_dir", NULL);
+			g_object_unref (G_OBJECT (gc));
+			if (path == NULL || strcmp (path, "") == 0) {
+				filename = g_build_filename (path, "image.iso.XXXXXX", NULL);
+				fd = mkstemp(filename);
+				close (fd);
+			}
+		} else if (!music_cd) {
 			filename = g_strdup (source_iso);
 		}
 	}
 
 	if (source_iso == NULL && !music_cd) {
-		/* TODO: Check for availible diskspace */
-		res = make_iso (filename, label);
-		if (!res) {
-			/* User cancelled or we had an error. */
-			goto out;
+		if (filename != NULL) {
+			res = make_iso (filename, label, FALSE);
+		} else {
+			res = RESULT_RETRY;
+		}
+
+		if (res == RESULT_RETRY) {
+			g_free (filename);
+			filename = g_build_filename (g_get_tmp_dir (), "image.iso.XXXXXX", NULL);
+			fd = mkstemp(filename);
+			close (fd);
+
+			res = make_iso (filename, label, FALSE);
+			if (res == RESULT_RETRY) {
+				g_free (filename);
+				filename = g_build_filename (g_get_home_dir (), ".image.iso.XXXXXX", NULL);
+				fd = mkstemp(filename);
+				close (fd);
+
+				res = make_iso (filename, label, TRUE);
+			}
+
+			if (!res) {
+				/* User cancelled or we had an error. */
+				goto out;
+			}
 		}
 	}
 
@@ -518,8 +827,12 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 		setup_close_button ();
 		gtk_main ();
 	} else {
+		if (!wait_for_insertion (rec, blank_cd)) {
+			return RESULT_CANCEL;
+		}
+
 		cdrecorder = cd_recorder_new ();
-		
+
 		g_signal_connect (G_OBJECT (cdrecorder),
 				  "progress-changed",
 				  G_CALLBACK (progress_changed_cb),
@@ -544,13 +857,22 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				  "insert-rwcd-request",
 				  G_CALLBACK (insert_rwcd_request_cb),
 				  NULL);
-		res = cd_recorder_write_iso (cdrecorder, rec, filename, speed, eject_cd, blank_cd, music_cd, dummy);
+		
+		if (music_cd) {
+			tracks = build_list_of_audio_tracks ();
+		} else {
+			Track *track;
+			track = g_new0 (Track, 1);
+			track->type = TRACK_TYPE_DATA;
+			track->data.filename = g_strdup (filename);
+			tracks = g_list_prepend (tracks, track);
+		}
+		res = cd_recorder_write_tracks (cdrecorder, rec, tracks, speed, eject_cd, blank_cd, dummy);
+
 		if (source_iso == NULL) {
 			unlink (filename);
 		}
 
-		g_print ("res: %d\n", res);
-		
 		if (res == RESULT_FINISHED) {
 			cd_progress_set_text (_("Complete"));
 			setup_close_button ();
@@ -564,6 +886,8 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 	
  out:
 	g_free (filename);
+	g_list_foreach (tracks, (GFunc)track_free, NULL);
+	g_list_free (tracks);
 	return res;
 }
 
@@ -574,7 +898,7 @@ lookup_current_recorder (GladeXML *xml)
 	GtkWidget *option_menu;
 	GtkWidget *item;
 	CDDrive *rec;
-	
+
 	option_menu = glade_xml_get_widget (xml, "target_optionmenu");
 	item = gtk_menu_get_active (GTK_MENU (GTK_OPTION_MENU (option_menu)->menu));
 	rec = g_object_get_data (G_OBJECT (item), "cd-drive");
@@ -594,8 +918,9 @@ refresh_dialog (GladeXML *xml)
 	GtkWidget *music_check;
 	GtkWidget *name_entry;
 	GList *l, *children;
+	GConfClient *gc;
 	char *name;
-	int i;
+	int i, default_speed;
 	CDDrive *rec;
 
 	/* Find active recorder: */
@@ -612,6 +937,10 @@ refresh_dialog (GladeXML *xml)
 	}
 	g_list_free (children);
 
+	gc = gconf_client_get_default ();
+	default_speed = gconf_client_get_int (gc, "/apps/nautilus-cd-burner/default_speed", NULL);
+	g_object_unref (G_OBJECT (gc));
+
 	item = gtk_menu_item_new_with_label (_("Maximum Possible"));
 	g_object_set_data (G_OBJECT (item), "speed", GINT_TO_POINTER (0));
 	gtk_widget_show (item);
@@ -625,7 +954,7 @@ refresh_dialog (GladeXML *xml)
 		g_free (name);
 	}
 	/* Default to automatic */
-	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), MIN (default_speed, default_speed));
 
 	/* Disable speed if max speed < 1 */
 	gtk_widget_set_sensitive (option_menu, rec->max_speed_write > 0);
@@ -848,6 +1177,19 @@ verify_iso (const char *filename, char **iso_label)
 	return TRUE;
 }
 
+static GdkPixbuf *
+my_gdk_pixbuf_new_from_stock (const char *name)
+{
+	GtkWidget *label;
+	GdkPixbuf *icon;
+
+	label = gtk_label_new ("");
+	icon = gtk_widget_render_icon (label, name, GTK_ICON_SIZE_BUTTON, NULL);
+	gtk_widget_destroy (label);
+
+	return icon;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -861,6 +1203,8 @@ main (int argc, char *argv[])
 	GnomeVFSFileSize size;
 	char *source_iso, *disc_name = NULL;
 	struct stat stat_buf;
+	GConfClient *gc;
+	GdkPixbuf *icon;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -943,6 +1287,9 @@ main (int argc, char *argv[])
 	gtk_widget_hide (dummy_check);
 #endif
 
+	icon = my_gdk_pixbuf_new_from_stock ("gtk-cdrom");
+	gtk_window_set_icon (GTK_WINDOW (dialog), icon);
+	gdk_pixbuf_unref (icon);
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	gtk_widget_hide (dialog);
@@ -978,6 +1325,9 @@ main (int argc, char *argv[])
 		option_menu = glade_xml_get_widget (xml, "speed_optionmenu");
 		item = gtk_menu_get_active (GTK_MENU (gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu))));
 		speed = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "speed"));
+		gc = gconf_client_get_default ();
+		gconf_client_set_int (gc, "/apps/nautilus-cd-burner/default_speed", speed, NULL);
+		g_object_unref (G_OBJECT (gc));
 
 		res = burn_cd (rec, source_iso, label, eject, blank, music, speed, dummy);
 
